#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""ECCO v4 Python: mds_io

This module includes utility routines for loading binary files in the llc 13-tile native flat binary layout.  This layout is the default for MITgcm input and output for global setups using lat-lon-cap (llc) layout.  The llc layout is used for ECCO v4. 

.. _ecco_v4_py Documentation :
   https://github.com/ECCO-GROUP/ECCOv4-py
"""

from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr
import time
from copy import deepcopy
import glob

#%%
def load_llc_mds(fdir, fname, llc, skip=0, nk=1, filetype = '>f', 
                 less_output = False ):
    """

    This routine loads a field from a MITgcm mds binary file in the llc 13 tie layout

    Parameters
    ----------
    fdir : string
        A string with the directory of the binary file to open
    fname : string
        A string with the name of the binary file to open
    llc : int
        the size of the llc grid.  For ECCO v4, we use the llc90 domain so `llc` would be `90`
    skip : int
        the number of 2D slices (or records) to skip.  Records could be vertical levels of a 3D field, or different 2D fields, or both.
    nk : int
        number of 2D slices (or records) to load.  
    filetype: string
        the file type, default is big endian (>) 32 bit float (f)
        alternatively, ('<d') would be little endian (<) 64 bit float (d)
    less_output : boolean
        a debug flag.  True means print more to the screen, False means be
        quieter.  Default False
        
    Returns
    -------
    ndarray
        the binary file contents organized into a llc x llc x 13 x nk `ndarray`, one llc x llc array for each of the 13 tiles

    Raises
    ------
    IOError
        If the file is not found

    """
    
    datafile = fdir + '/' + fname
    
    print 'loading ' + fname
    
        # check to see if file exists.    
    file = glob.glob(datafile)
    if len(file) == 0:
        raise IOError(fname + ' not found ')

    f = open(datafile, 'rb')
    dt = np.dtype(filetype)

    # skip ahead 'skip' number of 2D slices
    f.seek(llc*llc*13*skip*dt.itemsize)

    # read in 'nk' 2D slices (or records) from the mds file
    arr_k = np.fromfile(f, dtype=filetype, 
                        count=llc*llc*13*nk)
    
    f.close()
    
    # define a blank array
    arr_tiles_k = np.zeros((13, llc, llc, nk))
    
    #%%
    len_rec = 13*llc*llc

    # go through each 2D slice (or record)
    for k in  range(nk):

        tmp = arr_k[len_rec*(k):len_rec*(k+1)]
        arr = np.reshape(tmp,(13*llc, llc))
        
        f1 = arr[:3*llc,:]
        f2 = arr[3*llc:6*llc,:]
        f3 = arr[6*llc:7*llc,:]
        
        f4 = np.zeros((llc, 3*llc))
    
        for f in range(8,11):
            i1 = np.arange(0, llc)+(f-8)*llc
            i2 = np.arange(0,3*llc,3) + 7*llc + f -8
            f4[:,i1] = arr[i2,:]
    
        f5 = np.zeros((llc, 3*llc))
    
        for f in range(11,14):
            i1 = np.arange(0, llc)+(f-11)*llc
            i2 = np.arange(0,3*llc,3) + 10*llc + f -11
            f5[:,i1] = arr[i2,:]
        
        if 1 == 0:
            plt.close('all')
            plt.imshow(f1, origin='lower')
            plt.figure()
            plt.imshow(f2, origin='lower')   
            plt.show()
            
            plt.figure()
            plt.imshow(f3, origin='lower')
            plt.show()
            
            plt.figure()
            plt.imshow(f4, origin='lower')
            plt.show()
            
            plt.figure()
            plt.imshow(f5, origin='lower')
            plt.show()
            
        arr_tiles = np.zeros((13, llc, llc))
        
        arr_tiles[0,:] = f1[llc*0:llc*1,:]
        arr_tiles[1,:] = f1[llc*1:llc*2,:]
        arr_tiles[2,:] = f1[llc*2:,:]
    
        arr_tiles[3,:] = f2[llc*0:llc*1,:]
        arr_tiles[4,:] = f2[llc*1:llc*2,:]
        arr_tiles[5,:] = f2[llc*2:,:]
        
        arr_tiles[6,:] = f3
    
        arr_tiles[7,:] = f4[:,llc*0:llc*1]
        arr_tiles[8,:] = f4[:,llc*1:llc*2]
        arr_tiles[9,:] = f4[:,llc*2:]
        
        arr_tiles[10,:] = f5[:,llc*0:llc*1]
        arr_tiles[11,:] = f5[:,llc*1:llc*2]
        arr_tiles[12,:] = f5[:,llc*2:]
        
        if 1 == 0:
            plt.figure()
            plt.imshow(arr_tiles[0], origin='lower')
            plt.figure()
            plt.imshow(arr_tiles[1], origin='lower')
            plt.figure()
            plt.imshow(arr_tiles[2], origin='lower')
    
        arr_tiles_k[:,:,:,k] = arr_tiles
  
    if nk == 1:
        arr_tiles_k = arr_tiles_k[:,:,:,0]
        
    # return the array
    return arr_tiles_k



#!/usr/bin/env python2
# -*- coding: utf-8 -*-

from __future__ import division
import numpy as np
import matplotlib.pylab as plt
import xarray as xr

# import all function from the 'mpl_toolkits.basemap' module available with the 
# prefix 'Basemap'
from mpl_toolkits.basemap import Basemap

# Performs cartographic transformations and geodetic computations.

# The Proj class can convert from geographic (longitude,latitude) to native 
# map projection (x,y) coordinates and vice versa, or from one map projection 
# coordinate system directly to another.
# https://pypi.python.org/pypi/pyproj?
#
import pyresample as pr
import scipy.interpolate as interpolate
import math

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

def resample_to_latlon_nearest(orig_lons, orig_lats, orig_field, 
                     new_grid_min_lat, new_grid_max_lat, new_grid_delta_lat, 
                     new_grid_min_lon, new_grid_max_lon, new_grid_delta_lon,
                     nprocs_user) :
    
    

    #%%
    if type(orig_lats) == xr.core.dataarray.DataArray:
        orig_lons_1d = \
            orig_lons.values.reshape(np.product(orig_lons.values.shape))
        orig_lats_1d = \
            orig_lats.values.reshape(np.product(orig_lats.values.shape))   

    elif type(orig_lats) == np.ndarray:
        orig_lats_1d = orig_lats.reshape(np.product(orig_lats.shape))       
        orig_lons_1d = orig_lons.reshape(np.product(orig_lons.shape))
    else:
        print 'orig_lons and orig_lats variable either a DataArray or numpy.ndarray'
        print 'orig_lons found type ', type(orig_lons)
        print 'orig_lats found type ', type(orig_lats)        
        return
    
    if type(orig_field) == xr.core.dataarray.DataArray:
        orig_field = orig_field.values
    elif type(orig_field) != np.ndarray:
        print 'orig_fieldmust be either a DataArray or ndarray type \n'
        print 'found type ', type(orig_field)
        return

  
    
    # prepare for the nearest neighbor mapping
    
    # first define the lat lon points of the original data
    orig_grid = pr.geometry.SwathDefinition(lons=orig_lons_1d, 
                                            lats=orig_lats_1d)
    
    # the latitudes to which we will we interpolate

    # the latitudes to which we will we interpolate
    lat_tmp = (np.arange(new_grid_min_lat, new_grid_max_lat, 
                         new_grid_delta_lat))
    lon_tmp = (np.arange(new_grid_min_lon, new_grid_max_lon, 
                         new_grid_delta_lon))
        
    new_grid_lon, new_grid_lat = np.meshgrid(lon_tmp, lat_tmp)
    
    # define the lat lon points of the two parts. 
    new_grid  = pr.geometry.GridDefinition(lons=new_grid_lon, 
                                           lats=new_grid_lat)
    
    
    data_latlon_projection = \
            pr.kd_tree.resample_nearest(orig_grid, orig_field, new_grid, 
                                        radius_of_influence=1000000, 
                                        fill_value=None, 
                                        nprocs=nprocs_user) 

    #print 'nprocs_user ', nprocs_user
    
    return new_grid_lon, new_grid_lat, data_latlon_projection
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

